@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: plexer.ml,v 6.15 2012-01-09 14:22:21 deraugla Exp $ *)
+(* $Id: plexer.ml,v 6.17 2012-10-09 03:51:48 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 #load "pa_lexer.cmo";
@@ -56,6 +56,25 @@ value stream_peek_nth n strm =
 
 value utf8_lexing = ref False;
 
+value greek_tab =
+  ["α"; "β"; "γ"; "δ"; "ε"; "ζ"; "η"; "θ"; "ι"; "κ"; "λ"; "μ"; "ν"; "ξ";
+   "ο"; "π"; "ρ"; "σ"; "τ"; "υ"; "φ"; "χ"; "ψ"; "ω"]
+;
+
+value greek_letter buf strm =
+  if utf8_lexing.val then
+    match Stream.peek strm with
+    [ Some c ->
+        if Char.code c >= 128 then
+          let x = implode (Stream.npeek 2 strm) in
+          if List.mem x greek_tab then do { Stream.junk strm; $add c }
+          else raise Stream.Failure
+        else raise Stream.Failure
+    | None -> raise Stream.Failure ]
+  else
+    raise Stream.Failure
+;
+
 value misc_letter buf strm =
   if utf8_lexing.val then
     match Stream.peek strm with
@@ -370,6 +389,7 @@ value next_token_after_spaces ctx bp =
   [ 'A'-'Z' ident! ->
       let id = $buf in
       try ("", ctx.find_kwd id) with [ Not_found -> ("UIDENT", id) ]
+  | greek_letter ident! -> ("GIDENT", $buf)
   | [ 'a'-'z' | '_' | misc_letter ] ident! ->
       let id = $buf in
       try ("", ctx.find_kwd id) with [ Not_found -> ("LIDENT", id) ]
@@ -626,7 +646,7 @@ value using_token kwd_table ident_table (p_con, p_prm) =
             else Hashtbl.add ident_table p_prm p_con ]
   | "TILDEIDENT" | "TILDEIDENTCOLON" | "QUESTIONIDENT" |
     "QUESTIONIDENTCOLON" | "INT" | "INT_l" | "INT_L" | "INT_n" | "FLOAT" |
-    "CHAR" | "STRING" | "QUOTATION" |
+    "CHAR" | "STRING" | "QUOTATION" | "GIDENT" |
     "ANTIQUOT" | "ANTIQUOT_LOC" | "EOI" ->
       ()
   | _ ->
