@@ -1,5 +1,5 @@
 (* camlp5r pa_macro.cmo *)
-(* $Id: versdep.ml,v 6.19 2012-01-09 14:22:21 deraugla Exp $ *)
+(* $Id: versdep.ml,v 6.33 2012-06-01 18:34:02 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 open Parsetree;
@@ -46,8 +46,28 @@ value ocaml_location (fname, lnum, bolp, lnuml, bolpl, bp, ep) =
   END
 ;
 
+IFDEF OCAML_VERSION >= OCAML_4_00 THEN
+value loc_none =
+  let loc =
+    {Lexing.pos_fname = "_none_"; pos_lnum = 1; pos_bol = 0; pos_cnum = -1}
+  in
+  {Location.loc_start = loc;
+   Location.loc_end = loc;
+   Location.loc_ghost = True}
+;
+END;
+
+value mkloc loc txt =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN txt
+  ELSE {Location.txt = txt; loc = loc} END
+;
+value mknoloc txt =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN txt
+  ELSE mkloc loc_none txt END
+;
+
 value ocaml_id_or_li_of_string_list loc sl =
-  IFDEF OCAML_VERSION < OCAML_3_13_0 THEN
+  IFDEF OCAML_VERSION < OCAML_3_13_0 OR JOCAML THEN
     match List.rev sl with
     [ [s] -> Some s
     | _ -> None ]
@@ -74,6 +94,21 @@ value list_map_check f l =
     | [] -> Some (List.rev rev_l) ]
 ;
 
+value ocaml_value_description t p =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN {pval_type = t; pval_prim = p}
+  ELSE {pval_type = t; pval_prim = p; pval_loc = t.ptyp_loc} END
+;
+
+value ocaml_class_type_field loc ctfd =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN ctfd
+  ELSE {pctf_desc = ctfd; pctf_loc = loc} END
+;
+
+value ocaml_class_field loc cfd =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN cfd
+  ELSE {pcf_desc = cfd; pcf_loc = loc} END
+;
+
 value ocaml_type_declaration params cl tk pf tm loc variance =
   IFDEF OCAML_VERSION = OCAML_3_13_0_gadt THEN
     Right
@@ -107,13 +142,13 @@ value ocaml_type_declaration params cl tk pf tm loc variance =
           Right
             {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
              ptype_manifest = tm; ptype_loc = loc; ptype_variance = variance}
-        ELSIFDEF OCAML_VERSION < OCAML_3_13_0 THEN
+        ELSIFDEF OCAML_VERSION < OCAML_3_13_0 OR JOCAML THEN
           Right
             {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
              ptype_private = pf; ptype_manifest = tm; ptype_loc = loc;
              ptype_variance = variance}
         ELSE
-          let params = List.map (fun os -> Some os) params in
+          let params = List.map (fun os -> Some (mknoloc os)) params in
           Right
             {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
              ptype_private = pf; ptype_manifest = tm; ptype_loc = loc;
@@ -133,11 +168,25 @@ value ocaml_class_expr =
   ELSE Some (fun d loc -> {pcl_desc = d; pcl_loc = loc}) END
 ;
 
+value ocaml_class_structure p cil =
+  IFDEF OCAML_VERSION <= OCAML_4_00 THEN (p, cil)
+  ELSE {pcstr_pat = p; pcstr_fields = cil} END
+;
+
+value ocaml_pmty_ident loc li = Pmty_ident (mkloc loc li);
+
+value ocaml_pmty_functor sloc s mt1 mt2 = Pmty_functor (mkloc sloc s) mt1 mt2;
+
 value ocaml_pmty_typeof =
   IFDEF OCAML_VERSION < OCAML_3_12 THEN None
   ELSE Some (fun me -> Pmty_typeof me) END
 ;
 
+value ocaml_pmty_with mt lcl =
+  let lcl = List.map (fun (s, c) → (mknoloc s, c)) lcl in
+  Pmty_with mt lcl
+;
+
 value ocaml_ptype_abstract =
   IFDEF OCAML_VERSION <= OCAML_3_08_4 OR OCAML_VERSION >= OCAML_3_11 THEN
     Ptype_abstract
@@ -155,11 +204,14 @@ value ocaml_ptype_record ltl priv =
       let priv = if priv then Private else Public in
       Ptype_record ltl priv
     END
-  ELSIFDEF OCAML_VERSION >= OCAML_3_11 THEN
-    Ptype_record ltl
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_3_11 THEN
     let priv = if priv then Private else Public in
     Ptype_record ltl priv
+  ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
+    Ptype_record ltl
+  ELSE
+    Ptype_record
+      (List.map (fun (s, mf, ct, loc) → (mkloc loc s, mf, ct, loc)) ltl)
   END
 ;
 
@@ -190,7 +242,7 @@ value ocaml_ptype_variant ctl priv =
         in
           let priv = if priv then Private else Public in
           Some (Ptype_variant ctl priv)
-      ELSIFDEF OCAML_VERSION < OCAML_3_13_0 THEN
+      ELSIFDEF OCAML_VERSION < OCAML_3_13_0 OR JOCAML THEN
         let ctl =
           List.map
             (fun (c, tl, rto, loc) ->
@@ -202,7 +254,7 @@ value ocaml_ptype_variant ctl priv =
         let ctl =
           List.map
             (fun (c, tl, rto, loc) ->
-               if rto <> None then raise Exit else (c, tl, None, loc))
+               if rto <> None then raise Exit else (mknoloc c, tl, None, loc))
             ctl
         in
           Some (Ptype_variant ctl)
@@ -219,9 +271,11 @@ value ocaml_ptyp_arrow lab t1 t2 =
 
 value ocaml_ptyp_class li tl ll =
   IFDEF OCAML_VERSION <= OCAML_2_04 THEN Ptyp_class li tl
-  ELSE Ptyp_class li tl ll END
+  ELSE Ptyp_class (mknoloc li) tl ll END
 ;
 
+value ocaml_ptyp_constr li tl = Ptyp_constr (mknoloc li) tl;
+
 value ocaml_ptyp_package =
   IFDEF OCAML_VERSION < OCAML_3_12_0 THEN None
   ELSE Some (fun pt -> Ptyp_package pt) END
@@ -259,6 +313,10 @@ value ocaml_ptyp_variant catl clos sl_opt =
   END
 ;
 
+value ocaml_package_type li ltl =
+  (mknoloc li, List.map (fun (li, t) → (mkloc t.ptyp_loc li, t)) ltl)
+;
+
 value ocaml_const_int32 =
   IFDEF OCAML_VERSION <= OCAML_3_06 THEN None
   ELSE Some (fun s -> Const_int32 (Int32.of_string s)) END
@@ -319,6 +377,14 @@ value ocaml_pexp_assert fname loc e =
   ELSE Pexp_assert e END
 ;
 
+value ocaml_pexp_construct li po chk_arity =
+  Pexp_construct (mknoloc li) po chk_arity
+;
+
+value ocaml_pexp_field e li = Pexp_field e (mknoloc li);
+
+value ocaml_pexp_for i e1 e2 df e = Pexp_for (mknoloc i) e1 e2 df e;
+
 value ocaml_pexp_function lab eo pel =
   IFDEF OCAML_VERSION <= OCAML_2_04 THEN Pexp_function pel
   ELSE Pexp_function lab eo pel END
@@ -329,11 +395,15 @@ value ocaml_pexp_lazy =
   ELSE Some (fun e -> Pexp_lazy e) END
 ;
 
+value ocaml_pexp_ident li = Pexp_ident (mknoloc li);
+
 value ocaml_pexp_letmodule =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun i me e -> Pexp_letmodule i me e) END
+  ELSE Some (fun i me e -> Pexp_letmodule (mknoloc i) me e) END
 ;
 
+value ocaml_pexp_new loc li = Pexp_new (mkloc loc li);
+
 value ocaml_pexp_newtype =
   IFDEF OCAML_VERSION < OCAML_3_12_0 THEN None
   ELSE Some (fun s e -> Pexp_newtype s e) END
@@ -346,7 +416,12 @@ value ocaml_pexp_object =
 
 value ocaml_pexp_open =
   IFDEF OCAML_VERSION < OCAML_3_12 THEN None
-  ELSE Some (fun li e -> Pexp_open li e) END
+  ELSE Some (fun li e -> Pexp_open (mknoloc li) e) END
+;
+
+value ocaml_pexp_override sel =
+  let sel = List.map (fun (s, e) → (mknoloc s, e)) sel in
+  Pexp_override sel
 ;
 
 value ocaml_pexp_pack =
@@ -364,14 +439,19 @@ value ocaml_pexp_poly =
   ELSE Some (fun e t -> Pexp_poly e t) END
 ;
 
-value ocaml_pexp_record lel =
+value ocaml_pexp_record lel eo =
+  let lel = List.map (fun (li, loc, e) → (mkloc loc li, e)) lel in
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN
-    fun
+    match eo with
     [ Some _ -> invalid_arg "ocaml_pexp_record"
     | None -> Pexp_record lel ]
-  ELSE fun eo -> Pexp_record lel eo END
+  ELSE
+    Pexp_record lel eo
+  END
 ;
 
+value ocaml_pexp_setinstvar s e = Pexp_setinstvar (mknoloc s) e;
+
 value ocaml_pexp_variant =
   IFDEF OCAML_VERSION <= OCAML_2_04 THEN None
   ELSE
@@ -385,18 +465,26 @@ value ocaml_pexp_variant =
   END
 ;
 
+value ocaml_ppat_alias p i iloc = Ppat_alias p (mkloc iloc i);
+
 value ocaml_ppat_array =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
   ELSE Some (fun pl -> Ppat_array pl) END
 ;
 
-value ocaml_ppat_construct li po chk_arity  =
-  Ppat_construct li po chk_arity
+value ocaml_ppat_construct li li_loc po chk_arity  =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN
+    Ppat_construct li po chk_arity
+  ELSE
+    Ppat_construct (mkloc li_loc li) po chk_arity
+  END
 ;
 
 value ocaml_ppat_construct_args =
   fun
-  [ Ppat_construct li po chk_arity -> Some (li, po, chk_arity)
+  [ Ppat_construct li po chk_arity ->
+      IFDEF OCAML_VERSION < OCAML_4_00 THEN Some (li, 0, po, chk_arity)
+      ELSE Some (li.txt, li.loc, po, chk_arity) END 
   | _ -> None ]
 ;
 
@@ -405,22 +493,26 @@ value ocaml_ppat_lazy =
   ELSE None END
 ;
 
-value ocaml_ppat_record lpl closed =
-  IFDEF OCAML_VERSION >= OCAML_3_12 THEN
-    Ppat_record lpl (if closed then Closed else Open)
-  ELSE Ppat_record lpl END
+value ocaml_ppat_record lpl is_closed =
+  let lpl = List.map (fun (li, loc, p) → (mkloc loc li, p)) lpl in
+  IFDEF OCAML_VERSION < OCAML_3_12 THEN Ppat_record lpl
+  ELSE Ppat_record lpl (if is_closed then Closed else Open) END
 ;
 
 value ocaml_ppat_type =
   IFDEF OCAML_VERSION <= OCAML_2_99 THEN None
-  ELSE Some (fun sl -> Ppat_type sl) END
+  ELSE Some (fun loc li -> Ppat_type (mkloc loc li)) END
 ;
 
 value ocaml_ppat_unpack =
-  IFDEF OCAML_VERSION < OCAML_3_13_0 THEN None
-  ELSE Some (fun s -> Ppat_unpack s, fun pt -> Ptyp_package pt) END
+  IFDEF OCAML_VERSION < OCAML_3_13_0 OR JOCAML THEN None
+  ELSE
+    Some (fun loc s -> Ppat_unpack (mkloc loc s), fun pt -> Ptyp_package pt)
+  END
 ;
 
+value ocaml_ppat_var loc s = Ppat_var (mkloc loc s);
+
 value ocaml_ppat_variant =
   IFDEF OCAML_VERSION <= OCAML_2_04 THEN None
   ELSE
@@ -439,19 +531,42 @@ value ocaml_psig_class_type =
   ELSE Some (fun ctl -> Psig_class_type ctl) END
 ;
 
+value ocaml_psig_exception s ed = Psig_exception (mknoloc s) ed;
+
+value ocaml_psig_module s mt = Psig_module (mknoloc s) mt;
+
+value ocaml_psig_modtype s mtd = Psig_modtype (mknoloc s) mtd;
+
+value ocaml_psig_open li = Psig_open (mknoloc li);
+
 value ocaml_psig_recmodule =
   IFDEF OCAML_VERSION <= OCAML_3_06 THEN None
-  ELSE Some (fun ntl -> Psig_recmodule ntl) END
+  ELSE
+    let f ntl =
+      let ntl = List.map (fun (s, mt) → (mknoloc s, mt)) ntl in
+      Psig_recmodule ntl
+    in
+    Some f
+  END
 ;
 
+value ocaml_psig_type stl =
+  let stl = List.map (fun (s, t) → (mknoloc s, t)) stl in
+  Psig_type stl
+;
+
+value ocaml_psig_value s vd = Psig_value (mknoloc s) vd;
+
 value ocaml_pstr_class_type =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
   ELSE Some (fun ctl -> Pstr_class_type ctl) END
 ;
 
+value ocaml_pstr_exception s ed = Pstr_exception (mknoloc s) ed;
+
 value ocaml_pstr_exn_rebind =
   IFDEF OCAML_VERSION <= OCAML_2_99 THEN None
-  ELSE Some (fun s sl -> Pstr_exn_rebind s sl) END
+  ELSE Some (fun s li -> Pstr_exn_rebind (mknoloc s) (mknoloc li)) END
 ;
 
 value ocaml_pstr_include =
@@ -459,9 +574,32 @@ value ocaml_pstr_include =
   ELSE Some (fun me -> Pstr_include me) END
 ;
 
+value ocaml_pstr_modtype s mt = Pstr_modtype (mknoloc s) mt;
+
+value ocaml_pstr_module s me = Pstr_module (mknoloc s) me;
+
+value ocaml_pstr_open li = Pstr_open (mknoloc li);
+
+value ocaml_pstr_primitive s vd = Pstr_primitive (mknoloc s) vd;
+
 value ocaml_pstr_recmodule =
   IFDEF OCAML_VERSION <= OCAML_3_06 THEN None
-  ELSE Some (fun nel -> Pstr_recmodule nel) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
+    Some (fun nel -> Pstr_recmodule nel)
+  ELSE
+    let f nel =
+      Pstr_recmodule (List.map (fun (s, mt, me) → (mknoloc s, mt, me)) nel)
+    in
+    Some f
+  END
+;
+
+value ocaml_pstr_type stl = 
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN Pstr_type stl
+  ELSE
+    let stl = List.map (fun (s, t) → (mknoloc s, t)) stl in
+    Pstr_type stl
+  END
 ;
 
 value ocaml_class_infos =
@@ -473,12 +611,17 @@ value ocaml_class_infos =
           pci_expr = expr; pci_loc = loc})
   ELSE
     Some
-      (fun virt params name expr loc variance ->
-        {pci_virt = virt; pci_params = params; pci_name = name;
+      (fun virt (sl, sloc) name expr loc variance ->
+        let params = (List.map (fun s → mkloc loc s) sl, sloc) in
+        {pci_virt = virt; pci_params = params; pci_name = mkloc loc name;
          pci_expr = expr; pci_loc = loc; pci_variance = variance})
   END
 ;
 
+value ocaml_pmod_ident li = Pmod_ident (mknoloc li);
+
+value ocaml_pmod_functor s mt me = Pmod_functor (mknoloc s) mt me;
+
 value ocaml_pmod_unpack =
   IFDEF OCAML_VERSION < OCAML_3_12 THEN None
   ELSIFDEF OCAML_VERSION < OCAML_3_13_0 THEN
@@ -491,7 +634,9 @@ value ocaml_pmod_unpack =
 
 value ocaml_pcf_cstr =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun (t1, t2, loc) -> Pcf_cstr (t1, t2, loc)) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
+    Some (fun (t1, t2, loc) -> Pcf_cstr (t1, t2, loc))
+  ELSE Some (fun (t1, t2, loc) -> Pcf_constr (t1, t2)) END
 ;
 
 value ocaml_pcf_inher =
@@ -513,7 +658,8 @@ value ocaml_pcf_meth (s, pf, ovf, e, loc) =
   let pf = if pf then Private else Public in
   IFDEF OCAML_VERSION >= OCAML_3_12 THEN
     let ovf = if ovf then Override else Fresh in
-    Pcf_meth (s, pf, ovf, e, loc) 
+    IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_meth (s, pf, ovf, e, loc)
+    ELSE Pcf_meth (mkloc loc s, pf, ovf, e) END
   ELSE Pcf_meth (s, pf, e, loc) END
 ;
 
@@ -522,7 +668,8 @@ value ocaml_pcf_val (s, mf, ovf, e, loc) =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN Pcf_val (s, Public, mf, Some e, loc)
   ELSIFDEF OCAML_VERSION >= OCAML_3_12 THEN
     let ovf = if ovf then Override else Fresh in
-    Pcf_val (s, mf, ovf, e, loc)
+    IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_val (s, mf, ovf, e, loc)
+    ELSE Pcf_val (mkloc loc s, mf, ovf, e) END
   ELSE Pcf_val (s, mf, e,  loc) END
 ;
 
@@ -531,12 +678,18 @@ value ocaml_pcf_valvirt =
   ELSE
     let ocaml_pcf (s, mf, t, loc) =
       let mf = if mf then Mutable else Immutable in
-      Pcf_valvirt (s, mf, t, loc)
+      IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_valvirt (s, mf, t, loc)
+      ELSE Pcf_valvirt (mkloc loc s, mf, t) END
     in
     Some ocaml_pcf
   END
 ;
 
+value ocaml_pcf_virt (s, pf, t, loc) =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_virt (s, pf, t, loc)
+  ELSE Pcf_virt (mkloc loc s, pf, t) END
+;
+
 value ocaml_pcl_apply =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
   ELSIFDEF OCAML_VERSION <= OCAML_2_04 THEN
@@ -548,7 +701,7 @@ value ocaml_pcl_apply =
 
 value ocaml_pcl_constr =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun li ctl -> Pcl_constr li ctl) END
+  ELSE Some (fun li ctl -> Pcl_constr (mknoloc li) ctl) END
 ;
 
 value ocaml_pcl_constraint =
@@ -578,18 +731,32 @@ value ocaml_pcl_structure =
 
 value ocaml_pctf_cstr =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun (t1, t2, loc) -> Pctf_cstr (t1, t2, loc)) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
+    Some (fun (t1, t2, loc) -> Pctf_cstr (t1, t2, loc))
+  ELSE
+    Some (fun (t1, t2, loc) -> Pctf_cstr (t1, t2)) END
+;
+
+value ocaml_pctf_meth (s, pf, t, loc) =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN Pctf_meth (s, pf, t, loc)
+  ELSE Pctf_meth (s, pf, t) END
 ;
 
 value ocaml_pctf_val (s, mf, t, loc) =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN Pctf_val (s, Public, mf, Some t, loc)
-  ELSIFDEF OCAML_VERSION >= OCAML_3_10 THEN Pctf_val (s, mf, Concrete, t, loc)
-  ELSE Pctf_val (s, mf, Some t, loc) END
+  ELSIFDEF OCAML_VERSION < OCAML_3_10 THEN Pctf_val (s, mf, Some t, loc)
+  ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN Pctf_val (s, mf, Concrete, t, loc)
+  ELSE Pctf_val (s, mf, Concrete, t) END
+;
+
+value ocaml_pctf_virt (s, pf, t, loc) =
+  IFDEF OCAML_VERSION < OCAML_4_00 THEN Pctf_virt (s, pf, t, loc)
+  ELSE Pctf_virt (s, pf, t) END
 ;
 
 value ocaml_pcty_constr =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun li ltl -> Pcty_constr li ltl) END
+  ELSE Some (fun li ltl -> Pcty_constr (mknoloc li) ltl) END
 ;
 
 value ocaml_pcty_fun =
@@ -603,7 +770,17 @@ value ocaml_pcty_fun =
 
 value ocaml_pcty_signature =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun (t, cil) -> Pcty_signature (t, cil)) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
+    Some (fun (t, cil) -> Pcty_signature (t, cil))
+  ELSE
+    let f (t, ctfl) =
+      let cs =
+        {pcsig_self = t; pcsig_fields = ctfl; pcsig_loc = t.ptyp_loc}
+      in
+      Pcty_signature cs
+    in
+    Some f
+  END
 ;
 
 value ocaml_pdir_bool =
@@ -613,9 +790,11 @@ value ocaml_pdir_bool =
 
 value ocaml_pwith_modsubst =
   IFDEF OCAML_VERSION < OCAML_3_12_0 THEN None
-  ELSE Some (fun me -> Pwith_modsubst me) END
+  ELSE Some (fun loc me -> Pwith_modsubst (mkloc loc me)) END
 ;
 
+value ocaml_pwith_module loc me = Pwith_module (mkloc loc me);
+
 value ocaml_pwith_typesubst =
   IFDEF OCAML_VERSION < OCAML_3_12_0 THEN None
   ELSE Some (fun td -> Pwith_typesubst td) END
@@ -633,6 +812,55 @@ value has_records_with_with =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN False ELSE True END
 ;
 
+IFDEF JOCAML THEN
+  value joinclause (loc, jc) =
+    let jc =
+      List.map
+        (fun (loc, (jpl, e)) ->
+           let jpl =
+             List.map
+               (fun (locp, (loci, s), p) ->
+                  let ji = {pjident_desc = s; pjident_loc = loci} in
+                  {pjpat_desc = (ji, p); pjpat_loc = locp})
+               jpl
+           in
+           {pjclause_desc = (jpl, e); pjclause_loc = loc})
+        jc
+    in
+    {pjauto_desc = jc; pjauto_loc = loc}
+  ;
+END;
+
+value jocaml_pstr_def =
+  IFDEF JOCAML THEN Some (fun jcl -> Pstr_def (List.map joinclause jcl))
+  ELSE (None : option (_ -> _)) END
+;
+
+value jocaml_pexp_def =
+  IFDEF JOCAML THEN Some (fun jcl e -> Pexp_def (List.map joinclause jcl) e)
+  ELSE (None : option (_ -> _ -> _)) END
+;
+
+value jocaml_pexp_par =
+  IFDEF JOCAML THEN Some (fun e1 e2 -> Pexp_par e1 e2)
+  ELSE (None : option (_ -> _ -> _)) END
+;
+
+value jocaml_pexp_reply =
+  IFDEF JOCAML THEN
+    let pexp_reply loc e (sloc, s) =
+      let ji = {pjident_desc = s; pjident_loc = sloc} in
+      Pexp_reply e ji
+    in
+    Some pexp_reply
+  ELSE (None : option (_ -> _ -> _ -> _)) END
+;
+
+value jocaml_pexp_spawn =
+  IFDEF JOCAML THEN Some (fun e -> Pexp_spawn e)
+  ELSE (None : option (_ -> _)) END
+;
+
 value arg_rest =
   fun
   [ IFDEF OCAML_VERSION >= OCAML_2_00 THEN Arg.Rest r -> Some r END
