@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: pa_r.ml,v 6.41 2012-03-09 15:22:46 deraugla Exp $ *)
+(* $Id: pa_r.ml,v 6.44 2013-02-28 17:27:54 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 #load "pa_extend.cmo";
@@ -156,14 +156,6 @@ value ascii_of_greek s =
     | [] -> s ]
 ;
 
-(* should be added in lib/plexer.ml, perhaps, as a new token GREEK? *)
-value greek_token =
-  Grammar.Entry.of_parser gram "greek_token"
-    (parser
-       [: `("LIDENT", x) when List.exists (start_with x) greek_tab :] ->
-          ascii_of_greek x)
-;
-
 value warned = ref False;
 value warning_deprecated_since_6_00 loc =
   if not warned.val then do {
@@ -407,6 +399,7 @@ EXTEND
       | s = V STRING -> <:expr< $_str:s$ >>
       | s = V CHAR -> <:expr< $_chr:s$ >>
       | i = V LIDENT -> <:expr< $_lid:i$ >>
+      | i = V GIDENT -> <:expr< $_lid:i$ >>
       | i = V UIDENT -> <:expr< $_uid:i$ >>
       | "["; "]" -> <:expr< [] >>
       | "["; el = LIST1 expr SEP ";"; last = cons_expr_opt; "]" ->
@@ -486,6 +479,7 @@ EXTEND
       [ p1 = SELF; "."; p2 = SELF -> <:patt< $p1$ . $p2$ >> ]
     | "simple"
       [ s = V LIDENT -> <:patt< $_lid:s$ >>
+      | s = V GIDENT -> <:patt< $_lid:s$ >>
       | s = V UIDENT -> <:patt< $_uid:s$ >>
       | s = V INT -> <:patt< $_int:s$ >>
       | s = V INT_l -> <:patt< $_int32:s$ >>
@@ -541,6 +535,7 @@ EXTEND
           <:patt< { $_list:lpl$ } >>
       | "("; p = paren_ipatt; ")" -> p
       | s = V LIDENT -> <:patt< $_lid:s$ >>
+      | s = V GIDENT -> <:patt< $_lid:s$ >>
       | "_" -> <:patt< _ >> ] ]
   ;
   paren_ipatt:
@@ -577,7 +572,7 @@ EXTEND
   ;
   simple_type_parameter:
     [ [ "'"; i = ident -> Some i
-      | i = greek_token -> Some i
+      | i = GIDENT -> Some (ascii_of_greek i)
       | "_" -> None ] ]
   ;
   ctyp:
@@ -601,7 +596,7 @@ EXTEND
       [ t1 = SELF; "."; t2 = SELF -> <:ctyp< $t1$ . $t2$ >> ]
     | "simple"
       [ "'"; i = V ident "" -> <:ctyp< '$_:i$ >>
-      | i = greek_token -> <:ctyp< '$i$ >>
+      | i = GIDENT -> <:ctyp< '$ascii_of_greek i$ >>
       | "_" -> <:ctyp< _ >>
       | i = V LIDENT -> <:ctyp< $_lid:i$ >>
       | i = V UIDENT -> <:ctyp< $_uid:i$ >>
